Package: mlr3
Title: Machine Learning in R - Next Generation
Version: 1.7.1
Authors@R: 
  c(
    person("Michel", "Lang", , "michellang@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-9754-0393")),
    person("Bernd", "Bischl", , "bernd_bischl@gmx.net", role = "aut",
           comment = c(ORCID = "0000-0001-6002-6980")),
    person("Jakob", "Richter", , "jakob1richter@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-4481-5554")),
    person("Patrick", "Schratz", , "patrick.schratz@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-0748-6624")),
    person("Giuseppe", "Casalicchio", , "giuseppe.casalicchio@stat.uni-muenchen.de", role = "ctb",
           comment = c(ORCID = "0000-0001-5324-5966")),
    person("Stefan", "Coors", , "mail@stefancoors.de", role = "ctb",
           comment = c(ORCID = "0000-0002-7465-2146")),
    person("Quay", "Au", , "quayau@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-5252-8902")),
    person("Martin", "Binder", , "mlr.developer@mb706.com", role = "aut"),
    person("Florian", "Pfisterer", , "pfistererf@googlemail.com", role = "aut",
           comment = c(ORCID = "0000-0001-8867-762X")),
    person("Raphael", "Sonabend", , "raphaelsonabend@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-9225-4654")),
    person("Lennart", "Schneider", , "lennart.sch@web.de", role = "ctb",
           comment = c(ORCID = "0000-0003-4152-5308")),
    person("Marc", "Becker", , "marcbecker@posteo.de", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-8115-0400")),
    person("Sebastian", "Fischer", , "sebf.fischer@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-9609-3197")),
    person("Lona", "Koers", , "lona.koers@gmail.com", role = "ctb"),
    person("John", "Zobolas", , "bblodfon@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-3609-8674")),
    person("Maximilian", "Mücke", , "muecke.maximilian@gmail.com", role = "ctb",
           comment = c(ORCID = "0009-0000-9432-9795")),
    person("Keno", "Mersmann", , "keno.mersmann@gmail.com", role = "ctb")
  )
Description: Efficient, object-oriented programming on the
    building blocks of machine learning. Provides 'R6' objects for tasks,
    learners, resamplings, and measures. The package is geared towards
    scalability and larger datasets by supporting parallelization and
    out-of-memory data-backends like databases. While 'mlr3' focuses on
    the core computational operations, add-on packages provide additional
    functionality.
License: LGPL-3
URL: https://mlr3.mlr-org.com, https://github.com/mlr-org/mlr3
BugReports: https://github.com/mlr-org/mlr3/issues
Depends: R (>= 3.5.0)
Imports: R6 (>= 2.4.1), backports (>= 1.5.0), checkmate (>= 2.0.0),
        cli, data.table (>= 1.15.0), evaluate (>= 1.0.4), future,
        future.apply (>= 1.5.0), lgr (>= 0.3.4), mirai (>= 2.4.1),
        methods, mlbench, mlr3measures (>= 1.2.0), mlr3misc (>=
        0.21.0), parallelly, palmerpenguins, paradox (>= 1.0.1), uuid
Suggests: callr, codetools, datasets, future.callr, mlr3data,
        progressr, remotes, RhpcBLASctl, rpart, testthat (>= 3.3.0)
Encoding: UTF-8
Config/testthat/edition: 3
Config/testthat/parallel: false
NeedsCompilation: no
Collate: 'mlr_reflections.R' 'BenchmarkResult.R' 'CallbackResample.R'
        'ContextResample.R' 'warn_deprecated.R' 'DataBackend.R'
        'DataBackendCbind.R' 'DataBackendDataTable.R'
        'DataBackendRbind.R' 'DataBackendRename.R' 'HotstartStack.R'
        'Learner.R' 'LearnerClassif.R' 'mlr_learners.R'
        'LearnerClassifDebug.R' 'LearnerClassifFeatureless.R'
        'LearnerClassifRpart.R' 'LearnerRegr.R' 'LearnerRegrDebug.R'
        'LearnerRegrFeatureless.R' 'LearnerRegrRpart.R' 'Measure.R'
        'mlr_measures.R' 'MeasureAIC.R' 'MeasureBIC.R'
        'MeasureClassif.R' 'MeasureClassifCosts.R' 'MeasureDebug.R'
        'MeasureElapsedTime.R' 'MeasureInternalValidScore.R'
        'MeasureOOBError.R' 'MeasureRegr.R' 'MeasureRegrPinball.R'
        'MeasureRegrRQR.R' 'MeasureRegrRSQ.R'
        'MeasureSelectedFeatures.R' 'MeasureSimilarity.R'
        'MeasureSimple.R' 'Prediction.R' 'PredictionClassif.R'
        'PredictionData.R' 'PredictionDataClassif.R'
        'PredictionDataRegr.R' 'PredictionRegr.R' 'ResampleResult.R'
        'Resampling.R' 'mlr_resamplings.R' 'ResamplingBootstrap.R'
        'ResamplingCV.R' 'ResamplingCustom.R' 'ResamplingCustomCV.R'
        'ResamplingHoldout.R' 'ResamplingInsample.R' 'ResamplingLOO.R'
        'ResamplingRepeatedCV.R' 'ResamplingSubsampling.R'
        'ResultData.R' 'Task.R' 'TaskSupervised.R' 'TaskClassif.R'
        'mlr_tasks.R' 'TaskClassif_breast_cancer.R'
        'TaskClassif_german_credit.R' 'TaskClassif_iris.R'
        'TaskClassif_penguins.R' 'TaskClassif_pima.R'
        'TaskClassif_sonar.R' 'TaskClassif_spam.R' 'TaskClassif_wine.R'
        'TaskClassif_zoo.R' 'TaskGenerator.R' 'mlr_task_generators.R'
        'TaskGenerator2DNormals.R' 'TaskGeneratorCassini.R'
        'TaskGeneratorCircle.R' 'TaskGeneratorFriedman1.R'
        'TaskGeneratorMoons.R' 'TaskGeneratorPeak.R'
        'TaskGeneratorSimplex.R' 'TaskGeneratorSmiley.R'
        'TaskGeneratorSpirals.R' 'TaskGeneratorXor.R' 'TaskRegr.R'
        'TaskRegr_california_housing.R' 'TaskRegr_mtcars.R'
        'TaskUnsupervised.R' 'as_benchmark_result.R'
        'as_data_backend.R' 'as_learner.R' 'as_measure.R'
        'as_prediction.R' 'as_prediction_classif.R'
        'as_prediction_data.R' 'as_prediction_regr.R'
        'as_resample_result.R' 'as_resampling.R' 'as_result_data.R'
        'as_task.R' 'as_task_classif.R' 'as_task_regr.R'
        'as_task_unsupervised.R' 'assertions.R' 'auto_convert.R'
        'benchmark.R' 'benchmark_grid.R' 'bibentries.R'
        'default_fallback.R' 'default_measures.R' 'fix_factor_levels.R'
        'helper.R' 'helper_data_table.R' 'helper_exec.R'
        'helper_hashes.R' 'install_pkgs.R' 'marshal.R'
        'mlr_callbacks.R' 'mlr_sugar.R' 'mlr_test_helpers.R'
        'partition.R' 'predict.R' 'reexports.R' 'resample.R'
        'score_roc_measures.R' 'set_threads.R' 'set_validate.R'
        'task_converters.R' 'worker.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-11 11:56:25 UTC; marc
Author: Michel Lang [aut] (ORCID: <https://orcid.org/0000-0001-9754-0393>),
  Bernd Bischl [aut] (ORCID: <https://orcid.org/0000-0001-6002-6980>),
  Jakob Richter [aut] (ORCID: <https://orcid.org/0000-0003-4481-5554>),
  Patrick Schratz [aut] (ORCID: <https://orcid.org/0000-0003-0748-6624>),
  Giuseppe Casalicchio [ctb] (ORCID:
    <https://orcid.org/0000-0001-5324-5966>),
  Stefan Coors [ctb] (ORCID: <https://orcid.org/0000-0002-7465-2146>),
  Quay Au [ctb] (ORCID: <https://orcid.org/0000-0002-5252-8902>),
  Martin Binder [aut],
  Florian Pfisterer [aut] (ORCID:
    <https://orcid.org/0000-0001-8867-762X>),
  Raphael Sonabend [aut] (ORCID: <https://orcid.org/0000-0001-9225-4654>),
  Lennart Schneider [ctb] (ORCID:
    <https://orcid.org/0000-0003-4152-5308>),
  Marc Becker [cre, aut] (ORCID: <https://orcid.org/0000-0002-8115-0400>),
  Sebastian Fischer [aut] (ORCID:
    <https://orcid.org/0000-0002-9609-3197>),
  Lona Koers [ctb],
  John Zobolas [ctb] (ORCID: <https://orcid.org/0000-0002-3609-8674>),
  Maximilian Mücke [ctb] (ORCID: <https://orcid.org/0009-0000-9432-9795>),
  Keno Mersmann [ctb]
Maintainer: Marc Becker <marcbecker@posteo.de>
Repository: CRAN
Date/Publication: 2026-06-11 14:00:20 UTC
Built: R 4.6.0; ; 2026-06-11 15:53:42 UTC; unix
