# Generated by roxygen2: do not edit by hand

S3method(as.data.table,BenchmarkResult)
S3method(as.data.table,DictionaryLearner)
S3method(as.data.table,DictionaryMeasure)
S3method(as.data.table,DictionaryResampling)
S3method(as.data.table,DictionaryTask)
S3method(as.data.table,DictionaryTaskGenerator)
S3method(as.data.table,PredictionClassif)
S3method(as.data.table,PredictionRegr)
S3method(as.data.table,ResampleResult)
S3method(as.data.table,Resampling)
S3method(as.data.table,Task)
S3method(as_benchmark_result,BenchmarkResult)
S3method(as_benchmark_result,ResampleResult)
S3method(as_data_backend,DataBackend)
S3method(as_data_backend,data.frame)
S3method(as_learner,Learner)
S3method(as_learners,default)
S3method(as_learners,list)
S3method(as_measure,"NULL")
S3method(as_measure,Measure)
S3method(as_measures,"NULL")
S3method(as_measures,default)
S3method(as_measures,list)
S3method(as_prediction,Prediction)
S3method(as_prediction,PredictionDataClassif)
S3method(as_prediction,PredictionDataRegr)
S3method(as_prediction_classif,PredictionClassif)
S3method(as_prediction_classif,data.frame)
S3method(as_prediction_data,Prediction)
S3method(as_prediction_data,PredictionData)
S3method(as_prediction_data,list)
S3method(as_prediction_regr,PredictionRegr)
S3method(as_prediction_regr,data.frame)
S3method(as_predictions,list)
S3method(as_resample_result,ResampleResult)
S3method(as_resample_result,ResultData)
S3method(as_resample_result,list)
S3method(as_resampling,Resampling)
S3method(as_resamplings,default)
S3method(as_resamplings,list)
S3method(as_task,Task)
S3method(as_task,default)
S3method(as_task_classif,DataBackend)
S3method(as_task_classif,TaskClassif)
S3method(as_task_classif,TaskRegr)
S3method(as_task_classif,data.frame)
S3method(as_task_classif,formula)
S3method(as_task_classif,matrix)
S3method(as_task_regr,DataBackend)
S3method(as_task_regr,TaskClassif)
S3method(as_task_regr,TaskRegr)
S3method(as_task_regr,data.frame)
S3method(as_task_regr,formula)
S3method(as_task_regr,matrix)
S3method(as_task_unsupervised,DataBackend)
S3method(as_task_unsupervised,Task)
S3method(as_task_unsupervised,data.frame)
S3method(as_tasks,default)
S3method(as_tasks,list)
S3method(as_tasks_unsupervised,Task)
S3method(as_tasks_unsupervised,list)
S3method(c,BenchmarkResult)
S3method(c,Prediction)
S3method(c,PredictionDataClassif)
S3method(c,PredictionDataRegr)
S3method(c,ResampleResult)
S3method(check_prediction_data,PredictionDataClassif)
S3method(check_prediction_data,PredictionDataRegr)
S3method(col_info,DataBackend)
S3method(col_info,data.table)
S3method(create_empty_prediction_data,TaskClassif)
S3method(create_empty_prediction_data,TaskRegr)
S3method(data.table::format_col,condition)
S3method(data.table::format_list_item,condition)
S3method(default_fallback,Learner)
S3method(default_fallback,LearnerClassif)
S3method(default_fallback,LearnerRegr)
S3method(default_values,Learner)
S3method(default_values,LearnerClassifRpart)
S3method(default_values,LearnerRegrRpart)
S3method(extract_pkgs,BenchmarkResult)
S3method(extract_pkgs,R6)
S3method(extract_pkgs,ResampleResult)
S3method(extract_pkgs,character)
S3method(extract_pkgs,list)
S3method(filter_prediction_data,PredictionDataClassif)
S3method(filter_prediction_data,PredictionDataRegr)
S3method(fix_factor_levels,data.table)
S3method(head,Task)
S3method(is_missing_prediction_data,PredictionDataClassif)
S3method(is_missing_prediction_data,PredictionDataRegr)
S3method(marshal_model,classif.debug_model)
S3method(marshal_model,default)
S3method(marshal_model,learner_state)
S3method(partition,Task)
S3method(predict,Learner)
S3method(print,PredictionData)
S3method(print,benchmark_grid)
S3method(print,bmr_aggregate)
S3method(print,bmr_score)
S3method(print,marshaled)
S3method(print,roc_measures)
S3method(print,rr_score)
S3method(rd_info,Learner)
S3method(rd_info,Measure)
S3method(rd_info,Task)
S3method(set_threads,R6)
S3method(set_threads,default)
S3method(set_threads,list)
S3method(set_validate,Learner)
S3method(summary,Task)
S3method(tail,Task)
S3method(task_check_col_roles,Task)
S3method(task_check_col_roles,TaskClassif)
S3method(task_check_col_roles,TaskRegr)
S3method(task_check_col_roles,TaskSupervised)
S3method(task_check_col_roles,TaskUnsupervised)
S3method(unmarshal_model,classif.debug_model_marshaled)
S3method(unmarshal_model,default)
S3method(unmarshal_model,learner_state_marshaled)
export(BenchmarkResult)
export(CallbackResample)
export(ContextResample)
export(DataBackend)
export(DataBackendDataTable)
export(HotstartStack)
export(Learner)
export(LearnerClassif)
export(LearnerClassifDebug)
export(LearnerClassifFeatureless)
export(LearnerClassifRpart)
export(LearnerRegr)
export(LearnerRegrDebug)
export(LearnerRegrFeatureless)
export(LearnerRegrRpart)
export(Measure)
export(MeasureAIC)
export(MeasureBIC)
export(MeasureClassif)
export(MeasureClassifCosts)
export(MeasureDebugClassif)
export(MeasureElapsedTime)
export(MeasureInternalValidScore)
export(MeasureOOBError)
export(MeasureRegr)
export(MeasureRegrPinball)
export(MeasureRegrRQR)
export(MeasureRegrRSQ)
export(MeasureSelectedFeatures)
export(MeasureSimilarity)
export(Prediction)
export(PredictionClassif)
export(PredictionRegr)
export(ResampleResult)
export(Resampling)
export(ResamplingBootstrap)
export(ResamplingCV)
export(ResamplingCustom)
export(ResamplingCustomCV)
export(ResamplingHoldout)
export(ResamplingInsample)
export(ResamplingLOO)
export(ResamplingRepeatedCV)
export(ResamplingSubsampling)
export(ResultData)
export(Task)
export(TaskClassif)
export(TaskGenerator)
export(TaskGenerator2DNormals)
export(TaskGeneratorCassini)
export(TaskGeneratorCircle)
export(TaskGeneratorFriedman1)
export(TaskGeneratorMoons)
export(TaskGeneratorPeak)
export(TaskGeneratorSimplex)
export(TaskGeneratorSmiley)
export(TaskGeneratorSpirals)
export(TaskGeneratorXor)
export(TaskRegr)
export(TaskSupervised)
export(TaskUnsupervised)
export(as.data.table)
export(as_benchmark_result)
export(as_data_backend)
export(as_learner)
export(as_learners)
export(as_measure)
export(as_measures)
export(as_prediction)
export(as_prediction_classif)
export(as_prediction_data)
export(as_prediction_regr)
export(as_predictions)
export(as_resample_result)
export(as_resampling)
export(as_resamplings)
export(as_result_data)
export(as_task)
export(as_task_classif)
export(as_task_regr)
export(as_task_unsupervised)
export(as_tasks)
export(as_tasks_unsupervised)
export(assert_backend)
export(assert_benchmark_result)
export(assert_empty_ellipsis)
export(assert_has_backend)
export(assert_learnable)
export(assert_learner)
export(assert_learners)
export(assert_measure)
export(assert_measures)
export(assert_predictable)
export(assert_prediction)
export(assert_quantiles)
export(assert_resample_callback)
export(assert_resample_callbacks)
export(assert_resample_result)
export(assert_resampling)
export(assert_resamplings)
export(assert_row_ids)
export(assert_scorable)
export(assert_task)
export(assert_tasks)
export(assert_validate)
export(auto_convert)
export(benchmark)
export(benchmark_grid)
export(callback_resample)
export(check_prediction_data)
export(clbk)
export(clbks)
export(col_info)
export(convert_task)
export(create_empty_prediction_data)
export(data.table)
export(default_fallback)
export(default_measures)
export(deprecated_binding_old)
export(extract_pkgs)
export(filter_prediction_data)
export(install_pkgs)
export(is_marshaled_model)
export(is_missing_prediction_data)
export(learner_marshal)
export(learner_marshaled)
export(learner_unmarshal)
export(lrn)
export(lrns)
export(marshal_model)
export(mlr_callbacks)
export(mlr_learners)
export(mlr_measures)
export(mlr_reflections)
export(mlr_resamplings)
export(mlr_task_generators)
export(mlr_tasks)
export(msr)
export(msrs)
export(partition)
export(resample)
export(rsmp)
export(rsmps)
export(score_roc_measures)
export(set_threads)
export(set_validate)
export(task_check_col_roles)
export(tgen)
export(tgens)
export(tsk)
export(tsks)
export(uhash)
export(uhashes)
export(unmarshal_model)
export(warn_deprecated_old)
import(checkmate)
import(cli)
import(data.table)
import(mlr3misc)
import(palmerpenguins)
import(paradox)
importFrom(R6,R6Class)
importFrom(R6,is.R6)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(future,nbrOfWorkers)
importFrom(future,plan)
importFrom(graphics,plot)
importFrom(methods,formalArgs)
importFrom(mlr3misc,clbk)
importFrom(mlr3misc,clbks)
importFrom(mlr3misc,mlr_callbacks)
importFrom(parallelly,availableCores)
importFrom(stats,contr.treatment)
importFrom(stats,model.frame)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,weighted.mean)
importFrom(utils,bibentry)
importFrom(utils,data)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
importFrom(uuid,UUIDgenerate)
