# $characteristics works

    Code
      task
    Output
      
      -- <TaskClassif> (4601x58): HP Spam Detection ----------------------------------
      * Target: type
      * Properties: twoclass
      * Features (57):
        * dbl (57): address, addresses, all, business, capitalAve, capitalLong,
        capitalTotal, charDollar, charExclamation, charHash, charRoundbracket,
        charSemicolon, charSquarebracket, conference, credit, cs, data, direct, edu,
        email, font, free, george, hp, hpl, internet, lab, labs, mail, make, meeting,
        money, num000, num1999, num3d, num415, num650, num85, num857, order,
        original, our, over, parts, people, pm, project, re, receive, remove, report,
        table, technology, telnet, will, you, your
      * Characteristics: foo=1, bar=a
      * Target classes: spam (positive class, 39%), nonspam (61%)

