% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_learner.R
\name{as_learner}
\alias{as_learner}
\alias{as_learner.Learner}
\alias{as_learners}
\alias{as_learners.default}
\alias{as_learners.list}
\title{Convert to a Learner}
\usage{
as_learner(x, ...)

\method{as_learner}{Learner}(x, clone = FALSE, discard_state = FALSE, ...)

as_learners(x, ...)

\method{as_learners}{default}(x, ...)

\method{as_learners}{list}(x, ...)
}
\arguments{
\item{x}{(any)\cr
Object to convert.}

\item{...}{(any)\cr
Additional arguments.}

\item{clone}{(\code{logical(1)})\cr
If \code{TRUE}, ensures that the returned object is not the same as the input \code{x}.}

\item{discard_state}{(\code{logical(1)})
Whether to discard the state.}
}
\value{
\link{Learner}.
}
\description{
Convert object to a \link{Learner} or a list of \link{Learner}.
}
